@@ -9,9 +9,11 @@ use crate::rate_limiter::RateLimiter;
 /// except according to those terms.
 use async_trait::async_trait;
 use log::error;
+use std::future::Future;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::time::{Duration, Instant};
 use tokio::sync::mpsc::Sender;
+use tokio::time::timeout;
 
 static STOP_ON_FATAL: AtomicBool = AtomicBool::new(false);
 
@@ -19,6 +21,7 @@ static STOP_ON_FATAL: AtomicBool = AtomicBool::new(false);
 pub struct BenchRun {
     pub index: usize,
     bench_begin: Instant,
+    timeout: Option<Duration>,
     requests_sent: usize,
     max_requests: Option<usize>,
     max_duration: Option<Duration>,
@@ -38,16 +41,18 @@ impl BenchRun {
         index: usize,
         max_requests: usize,
         rate_limiter: RateLimiter,
+        timeout: Option<Duration>,
     ) -> Self {
-        Self::new(index, Some(max_requests), None, rate_limiter)
+        Self::new(index, Some(max_requests), None, rate_limiter, timeout)
     }
 
     pub fn from_duration_limit(
         index: usize,
         max_duration: Duration,
         rate_limiter: RateLimiter,
+        timeout: Option<Duration>,
     ) -> Self {
-        Self::new(index, None, Some(max_duration), rate_limiter)
+        Self::new(index, None, Some(max_duration), rate_limiter, timeout)
     }
 
     fn new(
@@ -55,6 +60,7 @@ impl BenchRun {
         max_requests: Option<usize>,
         max_duration: Option<Duration>,
         rate_limiter: RateLimiter,
+        timeout: Option<Duration>,
     ) -> Self {
         assert!(
             max_duration.is_some() || max_requests.is_some(),
@@ -64,6 +70,7 @@ impl BenchRun {
         Self {
             index,
             bench_begin: Instant::now(),
+            timeout,
             requests_sent: 0,
             max_requests,
             max_duration,
@@ -102,15 +109,23 @@ impl BenchRun {
                 break;
             }
 
-            let request_stats = bench_protocol_adapter.send_request(&client).await;
-            let fatal_error = request_stats.fatal_error;
+            let timed_request = self
+                .timed_operation(bench_protocol_adapter.send_request(&client))
+                .await;
 
-            metrics_channel
-                .try_send(request_stats)
-                .map_err(|e| {
-                    error!("Error sending metrics: {}", e);
-                })
-                .unwrap_or_default();
+            let fatal_error = match timed_request {
+                Ok(request_stats) => {
+                    let failed = request_stats.fatal_error;
+                    metrics_channel
+                        .try_send(request_stats)
+                        .map_err(|e| {
+                            error!("Error sending metrics: {}", e);
+                        })
+                        .unwrap_or_default();
+                    failed
+                }
+                Err(_) => true,
+            };
 
             if fatal_error {
                 STOP_ON_FATAL.store(true, Ordering::Relaxed);
@@ -120,6 +135,23 @@ impl BenchRun {
 
         Ok(())
     }
+
+    /// Each async operation must be time-bound.
+    pub async fn timed_operation<T: Future>(&self, f: T) -> Result<<T as Future>::Output, ()> {
+        if let Some(timeout_value) = self.timeout {
+            let result = timeout(timeout_value, f).await;
+
+            if let Ok(r) = result {
+                println!("No timeout: {:?}", timeout_value);
+                Ok(r)
+            } else {
+                println!("Timeout: {:?}", timeout_value);
+                Err(())
+            }
+        } else {
+            return Ok(f.await);
+        }
+    }
 }
 
 #[cfg(test)]
@@ -134,7 +166,8 @@ mod tests {
     use crate::metrics::BenchRunMetrics;
     use mockito::mock;
     use std::sync::atomic::Ordering;
-    use std::time::Instant;
+    use std::thread::sleep;
+    use std::time::{Duration, Instant};
 
     #[tokio::test]
     async fn test_send_load() {
@@ -179,6 +212,7 @@ mod tests {
                     .expect("RateLadderBuilder failed"),
             )
             .mode(Http(http_adapter.clone()))
+            .request_timeout(None)
             .build()
             .expect("BenchmarkConfig failed");
 
@@ -259,6 +293,7 @@ mod tests {
                     .build()
                     .expect("RateLadderBuilder failed"),
             )
+            .request_timeout(None)
             .mode(Http(http_adapter.clone()))
             .build()
             .expect("BenchmarkConfig failed");
@@ -279,4 +314,64 @@ mod tests {
         assert!(STOP_ON_FATAL.load(Ordering::Relaxed));
         assert!(bench_result.is_ok());
     }
+
+    #[tokio::test]
+    async fn test_send_load_with_timeout() {
+        let request_count = 100;
+
+        let _m = mock("GET", "/1")
+            .with_status(200)
+            .with_body_from_fn(|_| {
+                sleep(Duration::from_secs(10));
+                Ok(())
+            })
+            .with_header("content-type", "text/plain")
+            .expect(request_count)
+            .create();
+
+        let url = mockito::server_url().to_string();
+        println!("Url: {}", url);
+        let http_adapter = HttpBenchAdapterBuilder::default()
+            .request(
+                HttpRequestBuilder::default()
+                    .url(vec![format!("{}/1", url)])
+                    .build()
+                    .unwrap(),
+            )
+            .config(HttpClientConfigBuilder::default().build().unwrap())
+            .build()
+            .unwrap();
+
+        let benchmark_config: BenchmarkConfig = BenchmarkConfigBuilder::default()
+            .rate_ladder(
+                RateLadderBuilder::default()
+                    .start(request_count as f64)
+                    .end(request_count as f64)
+                    .rate_increment(None)
+                    .step_duration(None)
+                    .step_requests(Some(request_count))
+                    .build()
+                    .expect("RateLadderBuilder failed"),
+            )
+            .request_timeout(Some(Duration::from_millis(10)))
+            .mode(Http(http_adapter.clone()))
+            .build()
+            .expect("BenchmarkConfig failed");
+
+        let mut session = benchmark_config.clone().new_bench_session();
+
+        let bench_run_stats = BenchRunMetrics::new();
+
+        STOP_ON_FATAL.store(false, Ordering::Relaxed);
+
+        let bench_result = session
+            .next()
+            .expect("Must have runs")
+            .run(bench_run_stats)
+            .await;
+
+        // must stop on timeout, treated as fatal
+        assert!(STOP_ON_FATAL.load(Ordering::Relaxed));
+        assert!(bench_result.is_ok());
+    }
 }
